@@ -1726,7 +1726,15 @@ int PrintList(Context* cx, Node* goalscar, List* module)
 		return 0;
 	}
 
-	if ((n->Car()->kind() != LIST) && (n->Car() != Nil)) {
+	if (n->Car()->kind() == ATOM) { 
+		n->Car()->print(cx->ioout);
+		fprintf(cx->ioout, "\n");
+	
+		return 1;
+	}
+
+	if ((n->Car()->kind() != LIST) && (n->Car()->kind() != ATOM) 
+					&& (n->Car() != Nil)) {
 		syserr("usage : <printlist LIST> \n");
 		return 0;
 	}
@@ -1751,6 +1759,13 @@ int PrintListNL(Context* cx, Node* goalscar, List* module)
 		return 0;
 	}
 
+	if (n->Car()->kind() == ATOM) { 
+		n->Car()->print(cx->ioout);
+		fprintf(cx->ioout, "\n");
+	
+		return 1;
+	}
+
 	if ((n->Car()->kind() != LIST) && (n->Car() != Nil)) {
 		syserr("usage : <printlistnl LIST> \n");
 		return 0;
@@ -1900,26 +1915,27 @@ static int DoUnifyCommon(Context* cx, Node* goalscar, List* module,
 			if (g->Car()->kind() != PRED) {
 				g = g->Car();
 			}
-				rt = sysmodule(cx, g->Car(), Nil, Nil, 
+
+			rt = sysmodule(cx, g->Car(), Nil, Nil, 
 							module, r);
-				if (rt) {
-					if (r>0) {
-				    		if (TraceFlag) {
-							PrintNode("sys module: ", goalscar->Val(), "...success");
-						}
-					} else {
-					    if (TraceFlag) {
-						if (r == 0) {
-							PrintNode("sys module: ", goalscar->Val(), "...false");
-						} else {	// rval == -1
-							PrintNode("sys module: ", goalscar->Val(), "...unknown");
-						}
-					    }
-					    return r;
+			if (rt) {
+				if (r>0) {
+			    		if (TraceFlag) {
+						PrintNode("sys module: ", goalscar->Val(), "...success");
 					}
 				} else {
-					return -1;
+				    if (TraceFlag) {
+					if (r == 0) {
+						PrintNode("sys module: ", goalscar->Val(), "...false");
+					} else {	// rval == -1
+						PrintNode("sys module: ", goalscar->Val(), "...unknown");
+					}
+				    }
+				    return r;
 				}
+			} else {
+				return -1;
+			}
 			return 1;
 		}
 #endif
@@ -2236,12 +2252,12 @@ static int DoUnifyCommon(Context* cx, Node* goalscar, List* module,
 // <Unify module goals>
 int DoUnify(Context* cx, Node* goalscar, List* module)
 {
-	DoUnifyCommon(cx, goalscar, module, 0);
+	return DoUnifyCommon(cx, goalscar, module, 0);
 }
 
 int DoInherit(Context* cx, Node* goalscar, List* module)
 {
-	DoUnifyCommon(cx, goalscar, module, 1);
+	return DoUnifyCommon(cx, goalscar, module, 1);
 }
 
 
