@@ -1,5 +1,5 @@
 /*
- * built-in program copyright (C) 2009 -2010 H.Niwa
+ * built-in program copyright (C) 2009 - 2012 H.Niwa
  */
 
 /*
@@ -37,8 +37,10 @@ extern int builtin(Context* cx, Node* goalscar, Node* &goalscdr,
 extern int ListLength(Node* n);
 extern int getvar(Node* var, Node* mod, Node*& val);
 extern int DoUnify(Context* cx, Node* goalscar, List* module);
+extern int DoInherit(Context* cx, Node* goalscar, List* module);
 
 extern int IncludeSub(Context* cx, Node* goalscar, List* module, char* libname);
+extern int EventThrow(Node* ev);
 
 #endif // __BUIDLIN_H
 
