@@ -1,5 +1,5 @@
 /*
- * compiler program copyright (C) 2009 H.Niwa
+ * compiler program copyright (C) 2009 - 2012 H.Niwa
  */
 
 /*
@@ -27,7 +27,7 @@ extern int compmodule(Context* cx, Node* goalscar, Node* goalscdr,
 
 extern int CompCheckReserved(Context* cx, Node* goalscar, List* module);
 extern int CompGetVar(Context* cx, Node* goalscar, List* module);
-extern int CompSetVar(Context* cx, Node* goalscar, List* module);
+extern int CompAddVar(Context* cx, Node* goalscar, List* module);
 extern int CompCheckVar(Context* cx, Node* goalscar, List* module);
 extern int CompNewFunc(Context* cx, Node* goalscar, List* module);
 extern int CompEndFunc(Context* cx, Node* goalscar, List* module);
