@@ -1,5 +1,5 @@
 /*
- * generator module program copyright (C) 2009 - 2011 H.Niwa
+ * generator module program copyright (C) 2009 - 2012 H.Niwa
  */
 
 /*
@@ -599,11 +599,15 @@ int callpermpred(Node* out[], int len2, paramtype &pt)
 // PrintNode(n); //dummy for test
 
 	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	Node* env = Nil->Cons(Nil);
 	SetEnv(env, nvar);
 	((Undef*)nvar)->Set(n);
@@ -846,11 +850,15 @@ int callcombipred(Node* out[], int len2, paramtype &pt)
 // PrintNode(n); //dummy for test
 
 	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+ 	cx2->integral = cx->integral;
+                
 	Node* env = Nil->Cons(Nil);
 	SetEnv(env, nvar);
 	((Undef*)nvar)->Set(n);
