@@ -1,5 +1,5 @@
 /*
- * ncurses lib program copyright (C) 2009 H.Niwa
+ * ncurses lib program copyright (C) 2009 - 2012 H.Niwa
  */
 
 /*
@@ -35,8 +35,12 @@
 #include <setjmp.h>
 
 #ifndef __MINGW32__
+#ifdef __CYGWIN__
+#include <ncurses/ncurses.h>
+#else
 #include <ncurses.h>
 #endif
+#endif
 
 #include <string>
 #include <complex>
@@ -53,8 +57,6 @@
 #include "sysmodule.h"
 #include "ncurlib.h"
 
-extern void PushStack(Context* cx, Node* goals, Node* md, Node* env);
-extern int PopStack(Context* cx, Node* &goals, Node* &md, Node* &env);
 
 int ncurlibflag = 0;
 
@@ -250,12 +252,16 @@ int curs_startcurses(Context* cx, Node* goalscar, List* module)
 	ncurlibflag = 1;
 
 
-	Context *cx2 = new Context(module);
+	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+	
 	cxpush(cx2, gl);
 	cxpush(cx2, goalscar);
 	cxpush(cx2, module);
