@@ -44,13 +44,13 @@ using namespace std;
 #include "sysmodule.h"
 #include "winsock2util.h"
 
-const char *VERSION = "0.24.0";
+const char *VERSION = "0.26.0";
 void
 optver ()
 {
   extern std::string code;
   printf ("Descartes %s\t(code:%s) \n", VERSION, code.c_str ());
-  printf ("Copyright (C) 2009 - 2011 Hideyuki Niwa\n\n");
+  printf ("Copyright (C) 2009 - 2012 Hideyuki Niwa\n\n");
 }
 
 #define STARTRC "dstartrc"
@@ -281,10 +281,20 @@ main (int argc, char **argv)
 
   struct timeval tv;
   gettimeofday (&tv, NULL);
+  extern struct timeval gctv_last;
+  gettimeofday (&gctv_last, NULL);
+  extern struct timeval gctv_long_last;
+  gettimeofday (&gctv_long_last, NULL);
+
   srand (tv.tv_usec);
 #else
   struct timeval tv;
   gettimeofday (&tv, NULL);
+  extern struct timeval gctv_last;
+  gettimeofday (&gctv_last, NULL);
+  extern struct timeval gctv_long_last;
+  gettimeofday (&gctv_long_last, NULL);
+
   srandom (tv.tv_usec);
 #endif
 
