@@ -80,7 +80,6 @@ int ncore = 0;
 
 int ProcFlag = 0;
 
-
 int NewProc(Context* cx, Node* goalscar, List* module);
 int EachProc(Context* cx, Node* goalscar, List* module);
 int FirstNewProc(Context* cx, Node* goalscar, List* module);
@@ -94,6 +93,20 @@ extern int DoFirstForeach(Context* cx, Node* goalscar, List* module);
 
 extern FILE* MksTemp(char* templ);
 
+int PROCN = 1;
+int setprocn(int n) 
+{
+	if (n < 1) {
+		n = 1;
+	}
+	
+	if (n > MAXCORE) {
+		PROCN = MAXCORE;
+	} else {
+		PROCN = n;
+	}
+}
+
 
 #define MAXPATHLEN 4096
 
@@ -226,13 +239,15 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 
-		waitpid(spid, NULL, 0);
+		waitpid(spid, (int*)NULL, 0);
 
+		setprocn(1);
 		return 1;
 	}
 
 	pid_t pid = 1;
 	pid_t pidno=0;
+	setprocn(to_val-init_val+1);
 	pid_t savepid[to_val-init_val+1];
 	Node* env = Nil->Cons(Nil);
 	Context *cx2;
@@ -262,7 +277,7 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 					int j;
 					for (j = 0; j <= pidno; j++) {
 						if (savepid[j] != 0) {
-							if (waitpid(savepid[j], NULL, 
+							if (waitpid(savepid[j], (int*)NULL, 
 							    WNOHANG) == savepid[j]) {
 								savepid[j] = 0;
 								ncore--;
@@ -279,7 +294,7 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 				usleep(1);
 				for (j = 0; j < pidno; j++) {
 					if (savepid[j] != 0) {
-						if (waitpid(savepid[j], NULL, 
+						if (waitpid(savepid[j], (int*)NULL, 
 						    WNOHANG) == savepid[j]) {
 							savepid[j] = 0;
 							ncore--;
@@ -364,7 +379,7 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 			flg = 0;
 			for (i = 0; i < pidno; i++) {
 				if (savepid[i] != 0) {
-					if (waitpid(savepid[i], NULL, 
+					if (waitpid(savepid[i], (int*)NULL, 
 							WNOHANG) == savepid[i]) {
 						savepid[i] = 0;
 						ncore--;
@@ -476,8 +491,10 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 		
-		waitpid(spid, NULL, 0);
+		waitpid(spid, (int*)NULL, 0);
 		
+		setprocn(1);
+
 		return 1;
 	}
 
@@ -485,6 +502,7 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 
 	pid_t pid = 1;
 	pid_t pidno=0;
+	setprocn(ListLength(nlist));
 	pid_t savepid[ListLength(nlist)];
 	for (pidno = 0 ; nlist->kind() != ATOM; ) {
 		if (pid != 0) {
@@ -511,7 +529,7 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 					int j;
 					for (j = 0; j <= pidno; j++) {
 						if (savepid[j] != 0) {
-							if (waitpid(savepid[j], NULL, 
+							if (waitpid(savepid[j], (int*)NULL, 
 							    WNOHANG) == savepid[j]) {
 								savepid[j] = 0;
 								ncore--;
@@ -528,7 +546,7 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 				usleep(1);
 				for (j = 0; j < pidno; j++) {
 					if (savepid[j] != 0) {
-						if (waitpid(savepid[j], NULL, 
+						if (waitpid(savepid[j], (int*)NULL, 
 						    WNOHANG) == savepid[j]) {
 							savepid[j] = 0;
 							ncore--;
@@ -619,7 +637,7 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 			flg = 0;
 			for (i = 0; i < pidno; i++) {
 				if (savepid[i] != 0) {
-					if (waitpid(savepid[i], NULL, 
+					if (waitpid(savepid[i], (int*)NULL, 
 							WNOHANG) == savepid[i]) {
 						savepid[i] = 0;
 						ncore--;
@@ -764,13 +782,16 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 
-		waitpid(spid, NULL, 0);
+		waitpid(spid, (int*)NULL, 0);
+
+		setprocn(1);
 
 		return 1;
 	}
 
 	pid_t pid = 1;
 	pid_t pidno=0;
+	setprocn(to_val-init_val+1);
 	pid_t savepid[to_val-init_val+1];
 	Node* env = Nil->Cons(Nil);
 	Context *cx2;
@@ -800,7 +821,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 					int j;
 					for (j = 0; j <= pidno; j++) {
 						if (savepid[j] != 0) {
-						   if (waitpid(savepid[j], NULL, 
+						   if (waitpid(savepid[j], (int*)NULL, 
 						       WNOHANG) == savepid[j]) {
 						         savepid[j] = 0;
 							 ncore--;
@@ -813,7 +834,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 							 }
 							 for (k = 0; k <= pidno; k++) {
 							    if (savepid[k]) {
-							       waitpid(savepid[k], NULL, 0);
+							       waitpid(savepid[k], (int*)NULL, 0);
 							       savepid[k] = 0;
 							       ncore--;
 							    }
@@ -833,7 +854,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 				usleep(1);
 				for (j = 0; j < pidno; j++) {
 					if (savepid[j] != 0) {
-					   if (waitpid(savepid[j], NULL, 
+					   if (waitpid(savepid[j], (int*)NULL, 
 					       WNOHANG) == savepid[j]) {
 					          savepid[j] = 0;
 						  ncore--;
@@ -846,7 +867,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 						   }
 						   for (k = 0; k <= pidno; k++) {
 						      if (savepid[k]) {
-						         waitpid(savepid[k], NULL, 0);
+						         waitpid(savepid[k], (int*)NULL, 0);
 						         savepid[k] = 0;
 						         ncore--;
 						      }
@@ -948,7 +969,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 					}
 					for (j = 0; j < pidno; j++) {
 						if (savepid[j]) {
-							waitpid(savepid[j], NULL, 0);
+							waitpid(savepid[j], (int*)NULL, 0);
 							savepid[j] = 0;
 							ncore--;
 						}
@@ -1057,13 +1078,16 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 		
-		waitpid(spid, NULL, 0);
+		waitpid(spid, (int*)NULL, 0);
 		
+		setprocn(1);
+
 		return 1;
 	}
 
 	pid_t pid = 1;
 	pid_t pidno=0;
+	setprocn(ListLength(nlist));
 	pid_t savepid[ListLength(nlist)];
 	for (pidno = 0 ; nlist->kind() != ATOM; ) {
 		int f = 0;
@@ -1091,7 +1115,7 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 					int j;
 					for (j = 0; j <= pidno; j++) {
 						if (savepid[j] != 0) {
-						   if (waitpid(savepid[j], NULL, 
+						   if (waitpid(savepid[j], (int*)NULL, 
 						       WNOHANG) == savepid[j]) {
 						         savepid[j] = 0;
 							 ncore--;
@@ -1104,7 +1128,7 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 							 }
 							 for (k = 0; k <= pidno; k++) {
 							    if (savepid[k]) {
-							       waitpid(savepid[k], NULL, 0);
+							       waitpid(savepid[k], (int*)NULL, 0);
 							       savepid[k] = 0;
 							       ncore--;
 							    }
@@ -1124,7 +1148,7 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 				usleep(1);
 				for (j = 0; j < pidno; j++) {
 					if (savepid[j] != 0) {
-					   if (waitpid(savepid[j], NULL, 
+					   if (waitpid(savepid[j], (int*)NULL, 
 					       WNOHANG) == savepid[j]) {
 					          savepid[j] = 0;
 						  ncore--;
@@ -1137,7 +1161,7 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 						   }
 						   for (k = 0; k <= pidno; k++) {
 						      if (savepid[k]) {
-						         waitpid(savepid[k], NULL, 0);
+						         waitpid(savepid[k], (int*)NULL, 0);
 						         savepid[k] = 0;
 						         ncore--;
 						      }
@@ -1246,7 +1270,7 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 					}
 					for (j = 0; j < pidno; j++) {
 						if (savepid[j]) {
-							waitpid(savepid[j], NULL, 0);
+							waitpid(savepid[j], (int*)NULL, 0);
 							savepid[j] = 0;
 							ncore--;
 						}
