@@ -1,5 +1,5 @@
 /*
- * Unify pred program copyright (C) 2009 H.Niwa
+ * Unify pred program copyright (C) 2009 - 2012 H.Niwa
  */
 
 /*
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <string.h>
 #include <setjmp.h>
 
 #include <string>
@@ -47,7 +48,7 @@
 #include "compiler.h"
 
 /* base module list */
-List *Module = (List *) Nil->Cons(Nil);
+List *Module;
 
 /* enable/disable trace flag */
 int TraceFlag = 0;
@@ -58,13 +59,23 @@ int StepFlag = 0;
 /* enable/disable display result flag */
 int DResultFlag = 0;
 
+/* break flag */
+int BreakFlag = 0;
+
+/* throw flag */
+int ThrowFlag = 0;
+
 int CheckInherit(Node* var, Context* cx)
 {
 	Node*	n = cx->inherit;
-
+	int	num = 0;
+	
 	for ( ; n->kind() != ATOM; n=n->Cdr()) {
 		if (n->Car()->Eq(var)) {
-			return 1;
+			num++;
+			if (num > 10) {
+				return 1;
+			}
 		}
 	}
 	return 0;
@@ -105,7 +116,10 @@ int PopStack(Context * cx, Node* &goals, Node* &md, Node* &env)
     /* return an input position to the old position */
     long long seekp;
     ((Atom*)nseekp)->toInt(seekp);
-    fseek(cx->ioin, (long)seekp, 0);
+
+    if (seekp > 0) {
+	fseek(cx->ioin, (long)seekp, 0);
+    }
 
     cx->env_stack = cx->env_stack->Cdr();
 
@@ -290,7 +304,7 @@ int Unification(Node* pred1, Node* pred2, Node* env, Context* cx)
 /* It is the Unify processing that does not have context in an argument. */
 int Unify(Node* goals)
 {
-    Context *cx = new Context(Module);
+    Context *cx = new Context(Module, new Atom("WORLD"));
 
     int r = Unify(cx, goals);
 
@@ -329,7 +343,18 @@ int Unify(Context*& cx, Node* goals, List* module)
      * 'goto RETUNIFY' is equivalent to 'return rval'. 
      */
 CALLUNIFY:
-    extern int sigint_stop;
+    extern int sigint_flag;
+    if (sigint_flag) {
+    	sigint_flag = 0;
+    	EventThrow(mka("SIGINT"));
+    }
+
+    if (BreakFlag) {
+    	return 1;
+    }
+    if (ThrowFlag) {
+    	return 1;
+    }
     
     // check timeout
     CheckTime();
@@ -348,6 +373,8 @@ CALLUNIFY:
     goalscar = GetLeftNode(goals, goalscdr, level);
     goalscar = goalscar->Val();
 
+    ErrGoal = goalscar;
+    
 //PrintNode("GetLeftNode goals ", goals);
 //PrintNode("GetLeftNode goalscar ", goalscar);
 //PrintNode("GetLeftNode goalscdr ", goalscdr);
@@ -366,7 +393,7 @@ CALLUNIFY:
 	fflush(stdin);
 	PrintNode("goals : ", goals->Val());
 	printf("step ! (s: step, c: continue) \n");
-    	fgets(c, 2, stdin);
+    	char* rsc = fgets(c, 2, stdin);
 	fflush(stdin);
     	
 	if (c[0] == 's') {
@@ -446,12 +473,20 @@ CALLUNIFY:
 		    } else {
 		        return -1;
 		    }
+	        } else if (s == BREAKOP) {
+//printf("break !!\n");
+		    if (TraceFlag) {
+		    	printf("*** break ***\n");
+		    }
+	            rval = 1;
+	            BreakFlag = 1;
+		    goto RETUNIFY;
 	        } else if (s == BRKOP) {
 //printf("brk !!\n");
 	    	    char c[1024];
 
 		    PrintNode("brk ! (s: step, c: continue) \n", goals);
-    	    	    fgets(c, 1023, stdin);
+    	    	    char* rsc = fgets(c, 1023, stdin);
     	    	    c[1023] = 0;
 		    for (int i=0; i < 1023; i++) {
 			if ((c[i] == 's') || (c[i] == '\n')){
@@ -481,13 +516,14 @@ CALLUNIFY:
 //PrintNode("goalscar trace 1 ", goalscar);
 
 //PrintNode("Unify Pred : ", goalscar->Val());
-	    cxpush(cx, bd);
+//	    cxpush(cx, bd);
+            bd = Nil;
 	    cxpush(cx, md);
 	    cxpush(cx, goals);
 	    cxpush(cx, goalscar);
 	    cxpush(cx, goalscdr);
 	    if (builtin(cx, goalscar, goalscdr, goals, module, r)) {
-	    	cxpop(cx);
+//	    	cxpop(cx);
 	    	cxpop(cx);
 	    	cxpop(cx);
 	    	cxpop(cx);
@@ -524,7 +560,7 @@ CALLUNIFY:
 		}
 	    } else {
 //PrintNode("goalscar trace 1.5 ", goalscar);
-	    	cxpop(cx);
+//	    	cxpop(cx);
 	    	cxpop(cx);
 	    	cxpop(cx);
 	    	cxpop(cx);
@@ -541,26 +577,33 @@ CALLUNIFY:
     	std::string s;
     	((Atom*)goalscar)->toString(s);
 //	syserr("not a predication but strings:%s appeared. \n", s.c_str());
-#if 1
+
 	// token Node
-	extern int tokenCmpDirect(Context* cx, Node* goalscar, 
-					Node* goals, Node* md);
 	if (tokenCmpDirect(cx, goalscar, goalscdr, goals, md) == 1) {
 	    if (TraceFlag) {
-		PrintNode("token: ", goalscar->Val(), "...success");
+		PrintNode("token: [", goalscar->Val(), "]...success");
 	    }
 
 	    goals = goalscdr;
 	    goto CALLUNIFY;
 	} else {
+#if 0
 	    if (TraceFlag) {
-		PrintNode("token: ", goalscar->Val(), "...fail");
+		extern std::string RToken;
+
+	    	int tlen = RToken.length()+20;
+	    	char* tbuf = new char[tlen];
+	    	strncpy(tbuf, "token: [", 9);
+	    	strncat(tbuf, RToken.c_str(), RToken.length());
+	    	strncat(tbuf, "] : [", 6);
+		PrintNode(tbuf, goalscar->Val(), "]...fail");
 	    }
+#endif 
 
             rval = -1;
             goto RETUNIFY;
 	}
-#endif 
+
     }
 
 
@@ -572,7 +615,8 @@ CALLUNIFY:
     md = module->Car();
 //printf("Unify md "); md->print(); printf("\n");
 
-    for (;;) {
+    int undeclareflag;
+    for (undeclareflag=0;;undeclareflag++) {
 //PrintNode("goalscar trace 4 ", goalscar); fflush(stdout);
 	while (md->kind() == LIST) {
 //PrintNode("goalscar trace 5 ", goalscar); fflush(stdout);
@@ -595,7 +639,7 @@ CALLUNIFY:
 				cx->inherit = Cons(md->Car()->Cdr()->Car(), 
 						cx->inherit);
 //PrintNode("cx->inherit 2 ", cx->inherit);
-				rval = DoUnify(cx, MkList(mka("unify"), 
+				rval = DoInherit(cx, MkList(mka("unify"), 
 						md->Car()->Cdr()->Car(),
 						goalscar),
 					(List*)Module);
@@ -608,6 +652,8 @@ CALLUNIFY:
 			}
 //			md = md->Cdr();
 		} else {
+//PrintNode("name->Car() ", name->Car());
+//PrintNode("goalscar->Car() ", goalscar->Car());
 			if (ModuleCompare(name->Car(), goalscar->Car()) == 0) {
 				break;
 			}
@@ -616,16 +662,20 @@ CALLUNIFY:
 	}
 
 	if (md->kind() != LIST) {
-
+#if 0
+	    if (TraceFlag && (undeclareflag == 0)) {
+	    	PrintNode("Undeclared predicate : ", goalscar);
+	    }
+#endif
 //printf("Unify md==Nil call PopStack \n");
 	    if (PopStack(cx, goals, md, cx->env)) {
 		if (TraceFlag) {
 			PrintNode("*** back track ***2 ", goals->Car());
 		}
 //PrintNode("backtrack 2 goals ", goals);
-			if (goals->Car()->Eq(mka("x"))) {
-				xex(cx, goals->Cdr(), module);
-			}
+		if (goals->Car()->Eq(mka("x"))) {
+			xex(cx, goals->Cdr(), module);
+		}
 
 		level = 0;
 		goalscar = GetLeftNode(goals, goalscdr, level);
@@ -683,7 +733,7 @@ CALLUNIFY:
 
 RETUNIFY:
 	if (rval>0) {
-//printf("unify return rval \n");
+//printf("unify return rval %d\n", rval);
 		PushStack(cx, Nil, Nil, cx->env);
 		return rval;
 	} else if (rval == 0) {
