@@ -1,5 +1,5 @@
 /*
- * Var node program copyright (C) 2009 H.Niwa 
+ * Var node program copyright (C) 2009 - 2012 H.Niwa 
  */
 
 /*
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <string.h>
 
 #include <string>
 #include <complex>
@@ -39,12 +40,43 @@ unsigned int	undefvar = 0;
 Atom  __UNDEF__area("__UNDEF__");
 Node* __UNDEF__ = &__UNDEF__area;
 
+Node* FreeVarNodes = NULL;
+Node* FreeUndefNodes = NULL;
+
+Var* NewVar(std::string s)
+{
+	if (FreeVarNodes == NULL) {
+		return new Var(s);
+	} else {
+		Node* nd = FreeVarNodes;
+		FreeVarNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Var*)nd)->init(s);
+		return (Var*)nd;
+	}
+}
+
+static Undef* NewUndef()
+{
+	if (FreeUndefNodes == NULL) {
+		return new Undef();
+	} else {
+		Node* nd = FreeUndefNodes;
+		FreeUndefNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Undef*)nd)->init();
+		return (Undef*)nd;
+	}
+}
+
 
 
 Var::Var(std::string s)
 {
 	k = VAR;
-	varnode = new Undef();
+	varnode = NewUndef();
 	
 	name = s;
 }
@@ -128,6 +160,12 @@ int Var::SetForce(Node* n)
 
 }
 
+void Var::UnsetForce()
+{
+	varnode = NewUndef();
+	
+}
+
 void Var::gcmark()
 {
 	Node* np;
@@ -147,6 +185,15 @@ void Var::gcmark()
 	}
 }
 
+inline void Var::init(std::string s)
+{
+	//k = VAR;
+	varnode = NewUndef();
+	
+	name = s;
+}
+
+
 Undef::Undef() 
 {
 	k = UNDEF; 
@@ -183,3 +230,54 @@ void Undef::gcmark()
 	GCmark(value);
 }
 
+void Undef::UnsetForce()
+{
+	value = __UNDEF__; 
+}
+
+inline void Undef::init() 
+{
+	//k = UNDEF; 
+	udefno = undefvar++; 
+	value = __UNDEF__; 
+}
+
+
+/* Conversion of variable */
+
+Node*	varlist = Nil;
+
+void RegisterVar(Var* v)
+{
+	varlist = Cons(v, varlist);
+}
+
+Var* SearchVar(const char* s)
+{
+	Node* p;
+	for (p = varlist; p != Nil; p=p->Cdr()) {
+		if (((Var*)(p->Car()))->Name() == s) {
+			return ((Var*)p->Car());
+		}
+	}
+	return NULL;
+}
+
+Var* GetVar(const char* s)
+{
+	if (strncmp(s, "_", 1) != 0) {
+		Var* v = SearchVar(s);
+		if (v != NULL) {
+			return v;
+		}
+	}
+	Var* r = NewVar(s);
+	RegisterVar(r);
+	return r;
+}
+
+void ClearVar()
+{
+	varlist = Nil;
+}
+
